@@ -1,5 +1,5 @@
 // child.c (part of mintty)
-// Copyright 2008-11 Andy Koppe, 2015-2017 Thomas Wolff
+// Copyright 2008-11 Andy Koppe, 2015-2022 Thomas Wolff
 // Licensed under the terms of the GNU General Public License v3 or later.
 
 #include "child.h"
@@ -42,12 +42,17 @@ int forkpty(int *, char *, struct termios *, struct winsize *);
 
 string child_dir = null;
 
+bool logging = false;
 static pid_t pid;
 static bool killed;
 static int win_fd;
 static int pty_fd = -1;
 static int log_fd = -1;
-bool logging = false;
+#if CYGWIN_VERSION_API_MINOR >= 74
+static struct winsize prev_winsize = (struct winsize){0, 0, 0, 0};
+#else
+static struct winsize prev_winsize;
+#endif
 
 #if CYGWIN_VERSION_API_MINOR >= 66
 #include <langinfo.h>
@@ -215,11 +220,73 @@ child_update_charset(void)
 #endif
 }
 
+/*
+   Trim environment from variables set by launchers, other terminals, 
+   or other shells, in order to avoid confusing behaviour or invalid 
+   information (e.g. LINES, COLUMNS).
+ */
+static void
+trim_environment(void)
+{
+  void trimenv(string e) {
+    if (e[strlen(e) - 1] == '_') {
+#if CYGWIN_VERSION_API_MINOR >= 74
+      for (int ei = 0; environ[ei]; ei++) {
+        if (strncmp(e, environ[ei], strlen(e)) == 0) {
+          char * ee = strchr(environ[ei], '=');
+          if (ee) {
+            char * ev = strndup(environ[ei], ee - environ[ei]);
+            //printf("%s @%d - unsetenv %s: %s\n", e, ei, ev, environ[ei]);
+            unsetenv(ev);
+            free(ev);
+            // recheck current position after deletion
+            --ei;
+          }
+        }
+      }
+#endif
+    }
+    else
+      unsetenv(e);
+  }
+  // clear startup information from desktop launchers
+  trimenv("WINDOWID");
+  trimenv("GIO_LAUNCHED_");
+  trimenv("DESKTOP_STARTUP_ID");
+  // clear optional terminal configuration indications
+  trimenv("LINES");
+  trimenv("COLUMNS");
+  trimenv("TERMCAP");
+  trimenv("COLORFGBG");
+  trimenv("COLORTERM");
+  trimenv("DEFAULT_COLORS");
+  trimenv("WCWIDTH_CJK_LEGACY");
+  // clear identification from other terminals
+  trimenv("ITERM2_");
+  trimenv("MC_");
+  trimenv("PUTTY");
+  trimenv("RXVT_");
+  trimenv("URXVT_");
+  trimenv("VTE_");
+  trimenv("XTERM_");
+  trimenv("TERM_");
+  // clear indications from terminal multiplexers
+  trimenv("STY");
+  trimenv("WINDOW");
+  trimenv("TMUX");
+  trimenv("TMUX_PANE");
+  trimenv("BYOBU_");
+}
+
 void
 child_create(char *argv[], struct winsize *winp)
 {
   trace_dir(asform("child_create: %s", getcwd(malloc(MAX_PATH), MAX_PATH)));
 
+  trim_environment();
+
+  prev_winsize = *winp;
+
   // xterm and urxvt ignore SIGHUP, so let's do the same.
   signal(SIGHUP, SIG_IGN);
 
@@ -344,6 +411,10 @@ child_create(char *argv[], struct winsize *winp)
       printf("%d\n", pid);
       fflush(stdout);
     }
+    if (report_child_tty) {
+      printf("%s\n", ptsname(pty_fd));
+      fflush(stdout);
+    }
 
 #ifdef __midipix__
     // This corrupts CR in cygwin
@@ -404,6 +475,9 @@ child_tty(void)
 void
 child_proc(void)
 {
+  if (term.no_scroll)
+    return;
+
   for (;;) {
     if (term.paste_buffer)
       term_send_paste();
@@ -793,8 +867,10 @@ child_sendw(const wchar *ws, uint wlen)
 void
 child_resize(struct winsize *winp)
 {
-  if (pty_fd >= 0)
+  if (pty_fd >= 0 && memcmp(&prev_winsize, winp, sizeof(struct winsize)) != 0) {
+    prev_winsize = *winp;
     ioctl(pty_fd, TIOCSWINSZ, winp);
+  }
 }
 
 static int
@@ -803,13 +879,9 @@ foreground_pid()
   return (pty_fd >= 0) ? tcgetpgrp(pty_fd) : 0;
 }
 
-char *
-foreground_cwd()
+static char *
+get_foreground_cwd()
 {
-  // if working dir is communicated interactively, use it
-  if (child_dir && *child_dir)
-    return strdup(child_dir);
-
   // for WSL, do not check foreground process; hope start dir is good
   if (support_wsl) {
     char cwd[MAX_PATH];
@@ -830,6 +902,15 @@ foreground_cwd()
   return 0;
 }
 
+char *
+foreground_cwd()
+{
+  // if working dir is communicated interactively, use it
+  if (child_dir && *child_dir)
+    return strdup(child_dir);
+  return get_foreground_cwd();
+}
+
 char *
 foreground_prog()
 {
@@ -1016,7 +1097,7 @@ setenvi(char * env, int val)
 }
 
 static void
-setup_sync()
+setup_sync(bool in_tabs)
 {
   if (sync_level()) {
     if (win_is_fullscreen) {
@@ -1031,8 +1112,17 @@ setup_sync()
       setenvi("MINTTY_DX", r.right - r.left);
       setenvi("MINTTY_DY", r.bottom - r.top);
     }
-    if (cfg.tabbar)
+    if (cfg.tabbar) {
       setenvi("MINTTY_TABBAR", cfg.tabbar);
+      // enforce proper grouping, i.e. either new tab or window, as requested
+      if (in_tabs && *cfg.class) {
+        char * class = cs__wcstoutf(cfg.class);
+        setenv("MINTTY_CLASS", class, true);
+        free(class);
+      }
+      else if (!in_tabs)
+        setenv("MINTTY_CLASS", "+", true);
+    }
   }
 }
 
@@ -1040,10 +1130,9 @@ setup_sync()
   Called from Alt+F2 (or session launcher via child_launch).
  */
 static void
-do_child_fork(int argc, char *argv[], int moni, bool launch, bool config_size)
+do_child_fork(int argc, char *argv[], int moni, bool launch, bool config_size, bool in_cwd)
 {
   trace_dir(asform("do_child_fork: %s", getcwd(malloc(MAX_PATH), MAX_PATH)));
-  setup_sync();
 
 #ifdef control_AltF2_size_via_token
   void reset_fork_mode()
@@ -1077,15 +1166,21 @@ do_child_fork(int argc, char *argv[], int moni, bool launch, bool config_size)
   }
 
   if (clone == 0) {  // prepare child process to spawn new terminal
+    string set_dir = 0;
+    if (in_cwd)
+      set_dir = get_foreground_cwd(false);  // do this before close(pty_fd)!
+
     if (pty_fd >= 0)
       close(pty_fd);
     if (log_fd >= 0)
       close(log_fd);
     close(win_fd);
 
-    if (child_dir && *child_dir) {
-      string set_dir = child_dir;
-      if (support_wsl) {
+    if ((child_dir && *child_dir) || set_dir) {
+      if (set_dir) {
+        // use cwd of foreground process if requested via in_cwd
+      }
+      else if (support_wsl) {
         wchar * wcd = cs__utftowcs(child_dir);
 #ifdef debug_wsl
         printf("fork wsl <%ls>\n", wcd);
@@ -1097,23 +1192,26 @@ do_child_fork(int argc, char *argv[], int moni, bool launch, bool config_size)
         set_dir = (string)cs__wcstombs(wcd);
         delete(wcd);
       }
+      else
+        set_dir = strdup(child_dir);
 
-      chdir(set_dir);
-      trace_dir(asform("child: %s", set_dir));
-      setenv("PWD", set_dir, true);  // avoid softlink resolution
-      // prevent shell startup from setting current directory to $HOME
-      // unless cloned/Alt+F2 (!launch)
-      if (!launch) {
-        setenv("CHERE_INVOKING", "mintty", true);
-        // if cloned and then launched from Windows shortcut (!shortcut) 
-        // (by sanitizing taskbar icon grouping, #784, mintty/wsltty#96) 
-        // indicate to set proper directory
-        if (shortcut)
-          setenv("MINTTY_PWD", set_dir, true);
-      }
+      if (set_dir) {
+        chdir(set_dir);
+        trace_dir(asform("child: %s", set_dir));
+        setenv("PWD", set_dir, true);  // avoid softlink resolution
+        // prevent shell startup from setting current directory to $HOME
+        // unless cloned/Alt+F2 (!launch)
+        if (!launch) {
+          setenv("CHERE_INVOKING", "mintty", true);
+          // if cloned and then launched from Windows shortcut (!shortcut) 
+          // (by sanitizing taskbar icon grouping, #784, mintty/wsltty#96) 
+          // indicate to set proper directory
+          if (shortcut)
+            setenv("MINTTY_PWD", set_dir, true);
+        }
 
-      if (support_wsl)
         delete(set_dir);
+      }
     }
 
 #ifdef add_child_parameters
@@ -1150,6 +1248,11 @@ do_child_fork(int argc, char *argv[], int moni, bool launch, bool config_size)
     if (!config_size) {
       setenvi("MINTTY_ROWS", term.rows0);
       setenvi("MINTTY_COLS", term.cols0);
+#ifdef support_horizontal_scrollbar_with_tabbar
+      // this does not work, so horizontal scrollbar is disabled with tabbar
+extern int horsqueeze(void);  // should become horsqueeze_cols in win.h
+      setenvi("MINTTY_SQUEEZE", horsqueeze() / cell_width);
+#endif
       // provide environment to maximise window
       if (win_is_fullscreen)
         setenvi("MINTTY_MAXIMIZE", 2);
@@ -1196,9 +1299,12 @@ do_child_fork(int argc, char *argv[], int moni, bool launch, bool config_size)
   Called from Alt+F2.
  */
 void
-child_fork(int argc, char *argv[], int moni, bool config_size)
+child_fork(int argc, char *argv[], int moni, bool config_size, bool in_cwd, bool in_tabs)
 {
-  do_child_fork(argc, argv, moni, false, config_size);
+  setup_sync(in_tabs);
+  do_child_fork(argc, argv, moni, false, config_size, in_cwd);
+  // prevent wrong control of subsequent child_fork
+  unsetenv("MINTTY_CLASS");
 }
 
 /*
@@ -1222,7 +1328,7 @@ child_launch(int n, int argc, char * argv[], int moni)
         *sepp = '\0';
 
       if (n == 0) {
-        //setup_sync();
+        //setup_sync(false);
         argc = 1;
         char ** new_argv = newn(char *, argc + 1);
         new_argv[0] = argv[0];
@@ -1241,7 +1347,7 @@ child_launch(int n, int argc, char * argv[], int moni)
           }
         }
         new_argv[argc] = 0;
-        do_child_fork(argc, new_argv, moni, true, true);
+        do_child_fork(argc, new_argv, moni, true, true, false);
         free(new_argv);
         break;
       }
