@@ -9,10 +9,15 @@
 #include "winpriv.h"
 #include "charset.h"  // wcscpy
 
+#include "res.h"  // DIALOG_FONT, DIALOG_FONTSIZE
+
 #define _RPCNDR_H
 #define _WTYPES_H
 #define _OLE2_H
 #include <commdlg.h>
+#ifdef darken_dialog_elements
+#include <commctrl.h>  // Subclass
+#endif
 
 
 /*
@@ -35,14 +40,56 @@
 #define PUSHBTNHEIGHT 14
 #define PROGBARHEIGHT 14
 
+
+static HFONT _diafont = 0;
+static int diafontsize = 0;
+
+static void
+calc_diafontsize(void)
+{
+  cfg.options_fontsize = cfg.options_fontsize ?: DIALOG_FONTSIZE;
+
+  int fontsize = - MulDiv(cfg.options_fontsize, dpi, 72);
+  if (fontsize != diafontsize && _diafont) {
+    DeleteObject(_diafont);
+    _diafont = 0;
+  }
+  diafontsize = fontsize;
+}
+
+int
+scale_dialog(int x)
+{
+  calc_diafontsize();
+
+  return MulDiv(x, cfg.options_fontsize, DIALOG_FONTSIZE);
+  // scale by another * 1.2 for Comic Sans MS ...?
+}
+
+WPARAM
+diafont(void)
+{
+  calc_diafontsize();
+
+  if (!_diafont && (*cfg.options_font || cfg.options_fontsize != DIALOG_FONTSIZE))
+    _diafont = CreateFontW(
+                 diafontsize, 0, 0, 0,
+                 400, false, false, false,
+                 DEFAULT_CHARSET, OUT_DEFAULT_PRECIS,
+                 CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, 0,
+                 *cfg.options_font ? cfg.options_font : W(DIALOG_FONT)
+               );
+  return (WPARAM)_diafont;
+}
+
 void
 ctrlposinit(ctrlpos * cp, HWND wnd, int leftborder, int rightborder,
             int topborder)
 {
   RECT r, r2;
   cp->wnd = wnd;
-  cp->font = SendMessage(wnd, WM_GETFONT, 0, 0);
-  cp->ypos = topborder;
+  cp->font = diafont() ?: (WPARAM)SendMessage(wnd, WM_GETFONT, 0, 0);
+  cp->ypos = topborder;  // don't scale_dialog this; would move OK buttons out
   GetClientRect(wnd, &r);
   r2.left = r2.top = 0;
   r2.right = 4;
@@ -50,10 +97,26 @@ ctrlposinit(ctrlpos * cp, HWND wnd, int leftborder, int rightborder,
   MapDialogRect(wnd, &r2);
   cp->dlu4inpix = r2.right;
   cp->width = (r.right * 4) / (r2.right) - 2 * GAPBETWEEN;
-  cp->xoff = leftborder;
-  cp->width -= leftborder + rightborder;
+  // scale Options dialog items horizontally with custom font
+  cp->xoff = scale_dialog(leftborder);
+  cp->width -= cp->xoff + scale_dialog(rightborder);
 }
 
+#ifdef darken_dialog_elements
+static LRESULT CALLBACK
+ctl_proc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp, UINT_PTR uid, DWORD_PTR data)
+{
+  (void)uid; (void)data;
+  switch (msg) {
+    //when WM_ERASEBKGND:
+      // makes things worse (flickering, invisible items)
+    //when 0x0090 or 0x00F1 or 0x00F4 or 0x0143 or 0x014B:
+      // these also occur
+  }
+  return DefSubclassProc(hwnd, msg, wp, lp);
+}
+#endif
+
 static HWND
 doctl(control * ctrl, 
       ctrlpos * cp, RECT r, 
@@ -61,15 +124,24 @@ doctl(control * ctrl,
       string text, int wid)
 {
   HWND ctl;
+
  /*
   * Note nonstandard use of RECT. This is deliberate: by
   * transforming the width and height directly we arrange to
   * have all supposedly same-sized controls really same-sized.
   */
-
   r.left += cp->xoff;
   MapDialogRect(cp->wnd, &r);
 
+  // scale Options dialog items vertical position and size with custom font
+  r.top = scale_dialog(r.top);
+  r.bottom = scale_dialog(r.bottom);
+#if 0
+  // don't scale Options dialog items horizontally, done elsewhere
+  r.left = scale_dialog(cp->xoff) + scale_dialog(r.left - scale_dialog(cp->xoff));
+  r.right = scale_dialog(r.right);
+#endif
+
  /*
   * We can pass in cp->wnd == null, to indicate a dry run
   * without creating any actual controls.
@@ -82,8 +154,9 @@ doctl(control * ctrl,
       wchar * wtext = text ? cs__utftowcs(text) : 0;
       wchar * wclass = class ? cs__utftowcs(class) : 0;
       ctl =
-        CreateWindowExW(exstyle, wclass, wtext, wstyle, r.left, r.top, r.right,
-                        r.bottom, cp->wnd, (HMENU)(INT_PTR)wid, inst, null);
+        CreateWindowExW(exstyle, wclass, wtext, wstyle,
+                        r.left, r.top, r.right, r.bottom,
+                        cp->wnd, (HMENU)(INT_PTR)wid, inst, null);
       if (wtext)
         free(wtext);
       if (wclass)
@@ -91,9 +164,16 @@ doctl(control * ctrl,
     }
     else {
       ctl =
-        CreateWindowExA(exstyle, class, text, wstyle, r.left, r.top, r.right,
-                        r.bottom, cp->wnd, (HMENU)(INT_PTR)wid, inst, null);
+        CreateWindowExA(exstyle, class, text, wstyle,
+                        r.left, r.top, r.right, r.bottom,
+                        cp->wnd, (HMENU)(INT_PTR)wid, inst, null);
     }
+#ifdef darken_dialog_elements
+    // apply dark mode to dialog buttons
+    win_dark_mode(ctl);
+    // try to darken further elements
+    SetWindowSubclass(ctl, ctl_proc, 0, 0);
+#endif
 #ifdef debug_widgets
     printf("%8p %s %d '%s'\n", ctl, class, exstyle, text);
 #endif
@@ -519,6 +599,24 @@ staticddlbig(ctrlpos * cp, char *stext, int sid, int lid)
   cp->ypos += COMBOHEIGHT + GAPBETWEEN;
 }
 
+/*
+ * A static, text/label only.
+ */
+static void
+staticlabel(ctrlpos * cp, char *stext, int sid)
+{
+  RECT r;
+
+  if (stext) {
+    r.left = GAPBETWEEN;
+    r.top = cp->ypos;
+    r.right = cp->width;
+    r.bottom = STATICHEIGHT;
+    doctl(null, cp, r, "STATIC", WS_CHILD | WS_VISIBLE, 0, stext, sid);
+    cp->ypos += STATICHEIGHT + GAPBETWEEN;
+  }
+}
+
 /*
  * A list box with a static labelling it.
  */
@@ -652,10 +750,9 @@ winctrl_layout(winctrls *wc, ctrlpos *cp, controlset *s, int *id)
     * `ctrl' (a pointer to the control we have to create, or
     * think about creating, in this iteration of the loop),
     * `pos' (a suitable ctrlpos with which to position it), and
-    * `c' (a winctrl structure to receive details of the
-    * dialog IDs). Or we'll have done a `continue', if it was
-    * CTRL_COLUMNS and doesn't require any control creation at
-    * all.
+    * `c' (a winctrl structure to receive details of the dialog IDs).
+    * Or we'll have done a `continue', if it was CTRL_COLUMNS 
+    * and doesn't require any control creation at all.
     */
     if (ctrl->type == CTRL_COLUMNS) {
       assert((ctrl->columns.ncols == 1) ^ (ncols == 1));
@@ -806,6 +903,9 @@ winctrl_layout(winctrls *wc, ctrlpos *cp, controlset *s, int *id)
                              ctrl->listbox.ncols - 1, (LPARAM) tabarray);
         }
       }
+      when CTRL_LABEL:
+        num_ids = 1;
+        staticlabel(&pos, ctrl->label, base_id);
       when CTRL_FONTSELECT: {
         num_ids = 3;
         //__ Options - Text: font chooser activation button
@@ -972,10 +1072,18 @@ set_labels(bool font_chooser, int nCode, WPARAM wParam, LPARAM lParam)
     bool from_mouse = ((CBTACTIVATESTRUCT *)lParam)->fMouse;
     trace_hook(" mou %d\n", from_mouse);
   }
-  else
+  else {
     trace_hook("\n");
+  }
 #endif
 
+  void setfont(int id)
+  {
+    HWND button = GetDlgItem((HWND)wParam, id);
+    if (diafont())
+      SendMessage(button, WM_SETFONT, diafont(), MAKELPARAM(true, 0));
+  }
+
   void setlabel(int id, wstring label)
   {
     HWND button = GetDlgItem((HWND)wParam, id);
@@ -985,6 +1093,9 @@ set_labels(bool font_chooser, int nCode, WPARAM wParam, LPARAM lParam)
 #endif
       SetWindowTextW(button, label);
     }
+
+    // adjust custom font
+    setfont(id);
   }
 
   static int adjust_sample = 0;  /* pre-adjust (here) vs post-adjust (below)
@@ -1204,11 +1315,13 @@ set_labels(bool font_chooser, int nCode, WPARAM wParam, LPARAM lParam)
         GetWindowTextW(basic_colors, lbl, size);
       }
 
-      LRESULT fnt = SendMessage(basic_colors, WM_GETFONT, 0, 0);
+      WPARAM fnt = diafont() ?: (WPARAM)SendMessage(basic_colors, WM_GETFONT, 0, 0);
       DestroyWindow(basic_colors);
       //__ Colour chooser:
       basic_colors = CreateWindowExW(4, W("Static"), lbl ?: _W("B&asic colours:"), 0x50020000, 6, 7, 210, 15, (HWND)wParam, 0, inst, 0);
                          //shortkey disambiguated from original "&Basic colors:"
+      // this does not seem to apply dark mode to anything
+      //win_dark_mode(basic_colors);
       SendMessage(basic_colors, WM_SETFONT, fnt, MAKELPARAM(true, 0));
       if (lbl)
         free(lbl);
@@ -1315,6 +1428,25 @@ set_labels(bool font_chooser, int nCode, WPARAM wParam, LPARAM lParam)
     hooked_window_activated = true;
   }
 
+#ifdef debug_dlgitems
+    char * s = getenv("s");
+    int n;
+    while (s && (n = strtol(s, &s, 0))) {
+      printf("?%d\n", n);
+      setfont(n);
+    }
+#endif
+    // adjust custom font for anonymous system menu fields
+    setfont(703);  // Hue
+    setfont(704);  // Sat
+    setfont(705);  // Lum
+    setfont(706);  // Red
+    setfont(707);  // Green
+    setfont(708);  // Blue
+    setfont(1136);  // Font
+    setfont(1137);  // Font style
+    setfont(1138);  // Size
+
   if (do_next_hook) {
     trace_hook(" -> NextHook\n");
     LRESULT ret = CallNextHookEx(0, nCode, wParam, lParam);
@@ -1563,6 +1695,7 @@ dlg_text_paint(control *ctrl)
     SendMessage(wnd, WM_SETFONT, (WPARAM)fnt, MAKELPARAM(false, 0));
     SetWindowTextW(wnd, *new_cfg.font_sample ? new_cfg.font_sample : _W("Ferqœm’4€"));
   }
+  DeleteObject(fnt);
 }
 
 /*
@@ -1763,10 +1896,12 @@ void
 dlg_radiobutton_set(control *ctrl, int whichbutton)
 {
   winctrl *c = ctrl->plat_ctrl;
-  assert(c && c->ctrl->type == CTRL_RADIO);
-  CheckRadioButton(dlg.wnd, c->base_id + 1,
-                   c->base_id + c->ctrl->radio.nbuttons,
-                   c->base_id + 1 + whichbutton);
+  if (c) {  // can be null (leaving the dialog with ESC from another control)
+    assert(c && c->ctrl->type == CTRL_RADIO);
+    CheckRadioButton(dlg.wnd, c->base_id + 1,
+                     c->base_id + c->ctrl->radio.nbuttons,
+                     c->base_id + 1 + whichbutton);
+  }
 }
 
 int
